local VERSION = tonumber(("$Revision: 46033 $"):match("%d+"))

local Parrot = Parrot
if Parrot.revision < VERSION then
	Parrot.version = "r" .. VERSION
	Parrot.revision = VERSION
	Parrot.date = ("$Date: 2007-08-08 15:12:17 -0400 (Wed, 08 Aug 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local mod = Parrot:NewModule("Cooldowns", "AceEvent-2.0")

local L = Parrot:L("Parrot_Cooldowns")
local BS = AceLibrary("Babble-Spell-2.2")

local new, del = Parrot.new, Parrot.del

function mod:OnEnable()
	self:ResetSpells()
	
	self:ScheduleRepeatingEvent("Parrot_Cooldowns-cooldownOnUpdate", self.OnUpdate, 0.1, self)
	self:RegisterEvent("SPELLS_CHANGED", "ResetSpells")
	self:RegisterEvent("SPELL_UPDATE_COOLDOWN", "ResetCooldownState")
end

Parrot:RegisterCombatEvent{
	category = "Notification",
	subCategory = L["Cooldowns"],
	name = "Skill cooldown finish",
	localName = L["Skill cooldown finish"],
	defaultTag = L["[[Skill] ready!]"],
	tagTranslations = {
		Skill = 1,
		Icon = 2,
	},
	tagTranslationHelp = {
		Skill = L["The name of the spell or ability which is ready to be used."],
	},
	color = "ffffff", -- white
	sticky = false,
}

local cooldowns = {}
local spellNameToID = {}
local spellNameToTree = {}

function mod:ResetCooldownState()
	local GCD = 1.5
	if spellNameToID[BS["Shoot"]] then
		local _, shootCooldown = GetSpellCooldown(spellNameToID[BS["Shoot"]], "spell")
		if shootCooldown > GCD then
			GCD = shootCooldown
		end
	end
	for name, id in pairs(spellNameToID) do
		local start, duration = GetSpellCooldown(id, "spell")
		cooldowns[name] = start > 0 and duration > GCD
	end
end
function mod:ResetSpells()
	for k in pairs(spellNameToID) do
		spellNameToID[k] = nil
	end
	for k in pairs(cooldowns) do
		cooldowns[k] = nil
	end
	for i = 1, GetNumSpellTabs() do
		local _, _, offset, num = GetSpellTabInfo(i)
		for j = 1, num do
			local id = offset+j
			local spell = GetSpellName(id, "spell")
			spellNameToID[spell] = id
			spellNameToTree[spell] = i
		end
	end
	
	self:ResetCooldownState()
end

local groups = {
	[BS["Freezing Trap"]] = L["Traps"],
	[BS["Frost Trap"]] = L["Traps"],
	[BS["Immolation Trap"]] = L["Traps"],
	[BS["Snake Trap"]] = L["Traps"],
	[BS["Explosive Trap"]] = L["Traps"],
	[BS["Frost Shock"]] = L["Shocks"],
	[BS["Flame Shock"]] = L["Shocks"],
	[BS["Earth Shock"]] = L["Shocks"],
	[BS["Divine Shield"]] = BS["Divine Shield"],
	[BS["Divine Protection"]] = BS["Divine Shield"],
}

function mod:OnUpdate()
	local GCD = 1.5
	if spellNameToID[BS["Shoot"]] then
		local _, shootCooldown = GetSpellCooldown(spellNameToID[BS["Shoot"]], "spell")
		if shootCooldown > GCD then
			GCD = shootCooldown
		end
	end
	local groupsToTrigger = new()
	local spellsToTrigger = new()
	local treeCount = new()
	for name, id in pairs(spellNameToID) do
		local old = cooldowns[name]
		local start, duration = GetSpellCooldown(id, "spell")
		local check = start > 0 and duration > GCD
		cooldowns[name] = check
		if not check and old then
			spellsToTrigger[name] = id
			local tree = spellNameToTree[name]
			treeCount[tree] = (treeCount[tree] or 0) + 1
			Parrot:FirePrimaryTriggerCondition("Spell ready", name)
		end
	end
	for tree, num in pairs(treeCount) do
		if num >= 3 then
			for name in pairs(spellsToTrigger) do
				if tree == spellNameToTree[name] then
					spellsToTrigger[name] = nil
				end
			end
			local name, texture = GetSpellTabInfo(tree)
			local info = new(L["%s Tree"]:format(name), texture)
			Parrot:TriggerCombatEvent("Notification", "Skill cooldown finish", info)
			info = del(info)
		end
	end
	treeCount = del(treeCount)
	for name in pairs(spellsToTrigger) do
		if groups[name] then
			groupsToTrigger[groups[name]] = true
			spellsToTrigger[name] = nil
		end
	end
	for name in pairs(groupsToTrigger) do
		local info = new(name)
		Parrot:TriggerCombatEvent("Notification", "Skill cooldown finish", info)
		info = del(info)
	end
	groupsToTrigger = del(groupsToTrigger)
	for name, id in pairs(spellsToTrigger) do
		local info = new(name, GetSpellTexture(id, "spell"))
		Parrot:TriggerCombatEvent("Notification", "Skill cooldown finish", info)
		info = del(info)
	end
	spellsToTrigger = del(spellsToTrigger)
end

Parrot:RegisterPrimaryTriggerCondition {
	subCategory = L["Cooldowns"],
	name = "Spell ready",
	localName = L["Spell ready"],
	param = {
		type = 'text',
		usage = L["<Spell name>"],
	},
}

Parrot:RegisterSecondaryTriggerCondition {
	subCategory = L["Cooldowns"],
	name = "Spell ready",
	localName = L["Spell ready"],
	param = {
		type = 'text',
		usage = L["<Spell name>"],
	},
	check = function(param)
		return spellNameToID[param] and not cooldowns[param]
	end,
}
